# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright © 2011 Stephen Baumgartner <steve@slbaumgartner.com>
# Revisions Copyright 2016, Stephen Baumgartner and John McClenahan
#
# Permission to use, copy, modify, and distribute this software for
# any purpose and without fee is hereby granted, provided the above
# copyright notice appears in all copies.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------------
#
# This was extensively modified and enhanced from dim_angle.rb,
# Copyright 2005, Didier Bur, which, in turn, was based on the demo
# rectangle.rb by @Last Software
#
# Name:           draw_angle_dim
# Version         6.1
# Usage:          - Click 3 points, going around an angle in sequence from any point
#                 on one leg, to vertex, to any point on the other leg (clockwise or
#                 counterclockwise doesn't matter).  Be careful where you click -
#                 this tool lets the inference engine assist you but doesn't
#                 force you to click on existing model entities!
#                 - The tool will draw two edges along the sides of the angle,
#                 two part-arcs across the angle, and a 3Dtext showing the
#                 value of the angle in the gap.  The two edges make it easy to realign
#                 the group with the original angle if the entity containing it is
#                 subsequently moved or rotated.
#                 - The primitive elements of each dimension are gathered into a Group
#                 named Angular Dimension (<angle>), where <angle> is the measured
#                 angle.  If you draw multiple angle dimensions, the <angle> value
#                 will help you identify which is which (unless, of course, you have
#                 several with the same value!).
#                 - Each Angular Dimension drawing is packaged in an operation, so that
#                 undo and redo treat all of the elements as a single step.
#                 - The default radius for the arc is the distance between
#                 the first pick point and the vertex.  The user can override
#                 the radius by typing in the VCB at any time. If a new radius is entered
#                 before the first point is picked for the next angle, the prior
#                 dimension is redrawn at the new radius.  The user-selected
#                 radius remains in effect for additional angles until the
#                 user enters a new value or selects a different tool.
#                 - The TAB key (ALT/Option on Mac) toggles between drawing the interior (<180°) angle
#                 and drawing the exterior (>180°) angle dimension.  If you press TAB
#                 after an angle dimension is drawn but before picking the first
#                 point of the next angle, the current angle dimension will be
#                 redrawn in the new mode.
#                 - Unlike the built-in "smart" linear dimensions in SketchUp, the
#                 entities drawn by this tool are just graphics; they have NO tie
#                 to the original angle and will not follow it or scale if it is
#                 modified.  You can open the Group for edit, but it is not likely
#                 you will like the results - it is easier to delete the dimension
#                 Group and do it again.
# Date:           2017-01-23
# Type:           Tool
# Major Revisions:
#                 - 2017-02-23 - Added translations for Swedish and Russian
#                 - 2017-01-23 - Added locations for text and arrowheads for small angles
#                     when either arrowheads or text, but not both, will fit between dimension lines
#                     Also improved user feedback after setting Arc radius to zero in Measurments box.
#                     Completed and checked translations for five other latin-alphabet languages
#                 - 2016-12-18 - Added option to use square vs text for right angles
#                 - 2016-12-10 - Added detection to use comma instead of dot for decimal separator
#                 - 2016-11-01 - Extensively amended to place dimension text in plane of angle
#                   and add configurable user options in a web dialog
#                 - 2015-11-23 - Adapt for SU 2016 signing and for vector-graphic cursors
#                 and toolbar buttons.
#                 - 2013-09-07 - Enclose classes in my module.
#                 - 2012-10-31 - Added code to detect Mac ("darwin") systems
#                 and change to ALT key instead of TAB to change modes when on Mac.
#                 Note: it seems that Macs do not like .png files created on a PC
#                 and will fail to show cursors unless using native files.  Strange!
#                 Makes one wonder about the "portable" in PNG!
# Known Issues:   For some reason, the cursor sometimes does not appear on a Mac until the
#                 first time the user clicks after installing and activating the
#                 tool.  Subsequently it works fine, including if you quit and
#                 restart SketchUp.  This makes me think that on a Mac SU caches
#                 the cursors somewhere and doesn't get these until the tool is
#                 "really" active.
#                 Language translations not yet updated (2016-12)
# LanguageHandler added by Mario Chabot 2016. www.formation-sketchup.quebec
#-----------------------------------------------------------------------------
require 'sketchup.rb'
require 'extensions.rb'

module SLB_JWMPlugins
  module AngularDim

    # Constants to make the code independent of the Plugins folder
    BASEPATH = File.dirname(__FILE__).freeze
    EXTPATH = File.join(BASEPATH, 'SLB_JWM_draw_angle_dim').freeze
    IMGPATH = File.join(EXTPATH, 'Images').freeze
    RESPATH = File.join(EXTPATH, 'Resources').freeze

    # Message logging facility
    require File.join(EXTPATH, "logging.rb")
    # set true to enable logging during load of extension,
    # otherwise set false here and change later.
    AD_Logger = Logger.new(true)

    # create initial language strings translator.  This happens
    # before the rest of the extension is initialized.  As a
    # result, the registration info about the extension will
    # always be in the user's base locale or English if we
    # don't have a translation file for it.
    require File.join(EXTPATH, 'string_translate.rb')
    DangLH = StringTranslator.new(Sketchup.get_locale)

    # Load the extension.
    extension_name = DangLH['Angular Dimension 2']
    loader = File.join(EXTPATH, 'drawAngleDim_menu.rb')
    extension = SketchupExtension.new(extension_name, loader)
    extension.description = (DangLH['Create an angular dimension with']) + "\n" +
                            (DangLH[' arc and text in a new group in same plane as the angle.'])
    extension.version = '6.1'
    # constant based on code version for tagging of saved defaults
    EXTENSION_VERSION = extension.version

    extension.creator = 'Stephen Baumgartner and John McClenahan'
    extension.copyright = '2016-2017, slb.jwm@gmail.com.'

    Sketchup.register_extension extension, true


  end # AngularDim
end # SLB_JWMPlugins



