# SketchUp Tool callbacks for DrawAngleDimTool
# v6.1 2017-02-23 Steve Baumgartner and John McClenahan
#
# Implementation of the Tool callback protocol.
# In its own file to reduce clutter.
#

module SLB_JWMPlugins
  module AngularDim

    class DrawAngleDimTool

      #-----------------------------------------------------------------------------
      def activate
        reset
        self.class.active_tool = self
      end
      #-----------------------------------------------------------------------------
      def deactivate(view)
        reset
        # need to invalidate the view to cause any temporary geometry from the
        # draw method to be erased
        view.invalidate
        self.class.active_tool =  nil
      end
      #-----------------------------------------------------------------------------
      # need this to cause inference point and tooltips to be drawn
      def onMouseMove(flags, x, y, view)
        @ip.pick(view, x, y, @ip1)
        view.invalidate
        nil
      end
      #-----------------------------------------------------------------------------
      # user clicks the mouse button - capture the data point and advance the state
      def onLButtonDown(flags, x, y, view)
        set_current_point(x, y, view)
        increment_state
        nil
      end
      #-----------------------------------------------------------------------------
      def onCancel(flag, view)
        view.invalidate
        reset
      end
      #-----------------------------------------------------------------------------
      # Tell SketchUp that this Tool accepts user input via the VCB
      # Contrary to what the API docs say, SketchUp assumes a Tool will accept VCB
      # input by default and this method has no detectable effect!  Included here
      # in case they someday fix this bug.
      def enableVCB?
        true
      end
      #-----------------------------------------------------------------------------

      # accept user input in the VCB as the desired radius of the dimension arc
      def onUserText(text, view)
        # The user may type in something that we can't parse as a length
        # so we set up some exception handling to trap that
        begin
          value = text.to_l
        rescue
          # Error parsing the text
          UI.messagebox(DangLH["Please enter a length for arc radius"])
          value = nil
          Sketchup::set_status_text "", SB_VCB_VALUE
        end
        if(value < 0.0)
          UI.messagebox(DangLH["Arc radius must be a non-negative length"])
          value = nil
          Sketchup::set_status_text "", SB_VCB_VALUE
        end
        return if value.nil?

        self.class.user_radius = value
        # synchronize the settings dialog
        wd = self.class.settings_dialog
        if !wd.nil?
          self.class.pass_values_to_javascript(wd)
        end

        # redraw at new radius, if appropriate
        if @drawn && self.class.user_radius > 0.0
          # undo the previous operation before redrawing
          # this avoids buildup of obsolete operations on the
          # undo stack
          Sketchup.undo if @state==0
          draw_angle_dim
        end
        nil
      end

      # getExtents tells SketchUp to expand the model's bounding box to
      # include the temporary geometry we will create in the draw callback.
      # Without this, the temp geometry may get clipped.
      def getExtents
        bb=Sketchup.active_model.bounds
        case @state
        when 1
          bb.add( @pts[0], @ip.position)
        when 2
          bb.add(@pts[0], @pts[1], @ip.position)
        end
        bb
      end

      #-----------------------------------------------------------------------------
      # invoked by SketchUp when the view is invalidated.
      def draw(view)
        # make sure the input point's tooltip is displayed
        view.tooltip = @ip.tooltip
        @ip.draw view

        # provide visual feedback of what's been clicked.
        # note: SketchUp "forgets" the lines whenever the
        # view is invalidated.  They must be redrawn every
        # time or they will vanish.
        case @state
        when 1
          # first point selected,
          # draw "rubber band" to cursor from first point
          view.drawing_color = "magenta"
          view.line_width = 2
          view.draw(GL_LINE_STRIP,  @pts[0], @ip.position)
        when 2
          # first and second points selected,
          # draw line between first two points and "rubber band" from second to cursor
          view.drawing_color = "magenta"
          view.line_width = 2
          view.draw(GL_LINE_STRIP, @pts[0], @pts[1])
          view.draw(GL_LINE_STRIP, @pts[1], @ip.position)
        end
        nil
      end
      #-----------------------------------------------------------------------------
      # called when SketchUp thinks our cursor should be shown
      def onSetCursor
        UI::set_cursor(@cursor)
      end
      #-----------------------------------------------------------------------------
      # Toggle the drawing mode when the user presses TAB
      # on a PC or ALT on a Mac.  It is necessary to use
      # different keys per system because TAB on Mac and
      # ALT on PC are captured by the system to select
      # menu items and the keypress event never reaches here!
      def onKeyDown(key, rpt, flags, view)
        # platform sensitive mode toggle key
        if(RUBY_PLATFORM =~ /darwin/)
          toggle = VK_ALT # Mac Option/Alt key
        else
          # no virtual key code for tab on Windows?, so hard wired!
          toggle = 9
          # Use Control key on Windows until we find a better alternative
        end

        if key == toggle
          # inside vs outside toggle
          @inside = !@inside
          if @inside
            @cursor = IN_CURSOR
          else
            @cursor = OUT_CURSOR
          end
          UI.set_cursor(@cursor)

          # redraw in new mode if appropriate
          if @drawn
            # undo the previous operation before redrawing
            # this avoids buildup of unneeded operations on
            # the undo stack
            Sketchup.undo if @state==0
            draw_angle_dim
          end
        end
        nil
      end # onKeyDown
      #-----------------------------------------------------------------------------
    end # DrawAngleDimTool
  end # AngularDim
end # SLB_JWMPlugins
