# coding: utf-8
#
#  Data definitions, accessors, and initializers for the DrawAngleDimTool class
#   Modified 2016-10-09 JWM to add more parameters to InputBox, and change defaults for text height and arc radius
#   to 0.0 instead of -1 to check if user wants to override default text scale and picked radius
# Further attempted modifications to use web dialogue wdInputBox instead of native SU Ruby InputBox
#   2016-10-09 onwards. Syntax doesn't error, but the call to wdAngleDim_settings doesn't pop up the
#   web dialog, though calling from the Ruby console DOES pop it up.

# Version 6.1 2017-02-23 Stephen Baumgartner and John McClenahan

require File.join(SLB_JWMPlugins::AngularDim::EXTPATH, 'wdAngleDim_settings.rb')

module SLB_JWMPlugins
  module AngularDim

    class DrawAngleDimTool

      ###################
      # Class constants
      if(Sketchup.version.to_i >= 16)
        if(RUBY_PLATFORM =~ /darwin/)
          IN_ICON =  File.join(IMGPATH, 'inside.pdf') unless defined? IN_ICON
          OUT_ICON =  File.join(IMGPATH, 'outside.pdf') unless defined? OUT_ICON
        else
          IN_ICON =  File.join(IMGPATH, 'inside.svg') unless defined? IN_ICON
          OUT_ICON =  File.join(IMGPATH, 'outside.svg') unless defined? OUT_ICON
        end
      else
        IN_ICON =  File.join(IMGPATH, 'interior2.png') unless defined? IN_ICON
        OUT_ICON =  File.join(IMGPATH, 'exterior2.png') unless defined? OUT_ICON
      end

      IN_CURSOR = UI::create_cursor(IN_ICON, 0, 31) unless defined? IN_CURSOR
      OUT_CURSOR = UI::create_cursor(OUT_ICON, 0, 31) unless defined? OUT_CURSOR

      #################################
      # Class singleton variables and methods.  They remember values between invocations
      # of the tool.  CSV's are in some ways similar to @@ variables, in that they
      # belong to the class, not to instances, but are not inherited.  Hence many
      # Ruby experts consider them safer than @@ variables.

      DEFAULT_ARROW_STYLE = 'closed' unless defined? DEFAULT_ARROW_STYLE
      # Note: this expects the caller to have validated that style is
      # one of the supported choices.
      def self.arrow_style=(style)
        @arrow_style = style
      end
      def self.arrow_style
        @arrow_style
      end

      DEFAULT_ARROW_SCALE = 0.05 unless defined? DEFAULT_ARROW_SCALE
      def self.arrow_scale=(scale)
        @arrow_scale = scale.to_f
      end
      def self.arrow_scale
        @arrow_scale
      end

      DEFAULT_RADIUS = 0.0.to_l unless defined? DEFAULT_RADIUS
      def self.user_radius=(radius)
        radius = radius.to_l
        if radius > 0
          @user_radius = radius
        else
          @user_radius = DEFAULT_RADIUS
        end
      end
      def self.user_radius
        @user_radius
      end

      DEFAULT_ARC_SEGMENTS = 12 unless defined? DEFAULT_ARC_SEGMENTS
      def self.arc_segments=(segs)
        @arc_segments = segs.to_i
      end
      def self.arc_segments
        @arc_segments
      end

      DEFAULT_LINE_SCALE = 1.05 unless defined? DEFAULT_LINE_SCALE
      def self.dim_line_scale=(scale)
        @dim_line_scale = scale.to_f
      end
      def self.dim_line_scale
        @dim_line_scale
      end

      DEFAULT_TEXT_SCALE = 0.10 unless defined? DEFAULT_TEXT_SCALE
      def self.text_scale=(scale)
        @text_scale = scale.to_f
      end
      def self.text_scale
        @text_scale
      end

      DEFAULT_TEXT_HEIGHT = 0.0.to_l unless defined? DEFAULT_TEXT_HEIGHT
      def self.user_text_height=(height)
        height = height.to_l
        if height > 0.0
          @user_text_height = height
        else
          @user_text_height = DEFAULT_TEXT_HEIGHT
        end
      end
      def self.user_text_height
        @user_text_height
      end

      # Mark 90 degree corners with square mark instead of 90 degrees?
      DEFAULT_SQUARE90 = "yes" unless defined? DEFAULT_SQUARE90
      def self.square90=(square90)
        @square90 = square90
      end
      def self.square90
        @square90
      end

      # The following assumes we don't foolishly put files or folders with names
      # matching the pattern in Resources unless they are language handling folders!
      def self.scan_for_languages()
        # make sure the general default of en-US is in the list even though it
        # doesn't need a translation folder, otherwise it won't be available as
        # a choice in the inputbox!
        languages = ["en-US"]
        Dir.open(RESPATH) do |dir|
          dir.each {|name|
            languages << name if name =~ /^\w\w(-\w\w)?$/
          }
        end
        languages.uniq!
      end

      DEFAULT_LANGUAGE = Sketchup.get_locale() unless defined? DEFAULT_LANGUAGE
      @language = DEFAULT_LANGUAGE
      AVAILABLE_LANGUAGES =  self.scan_for_languages() unless defined? AVAILABLE_LANGUAGES

      # Name lookup hashes for all languages supported by SketchUp.
      # Notes:
      # - The list was generated from two sources: the SketchUp downloads page,
      # and the Ruby API documentation for Sketchup#get_locale().  These sources
      # each have some entries that are missing in the other, so I merged the lists.
      # There are no contradictions, so merging the lists harmlessly covers cases that
      # might not actually occur or that occurred only in obsolete SU versions.
      # For example, Dutch (nl) is not offered in SU 2016 or 2017.  I don't know
      # whether or when it was offered, but it does no harm to include it here.
      # - Not all of these languages are yet supported by this extension, but all known
      # codes in SketchUp are listed here.  Because the AVAILABLE_LANGUAGES
      # Array is populated by scanning the extension's Resources folder, this
      # extension will only offer those translations that are actually installed
      # in the extension.  Having a full list here means new translations can be
      # added simply by putting another folder into Resources without editing
      # the extension code.  If we get help from a native speaker to create the
      # translation lookup file for another language, this will let us send back
      # the translated files for testing without needing another incremental release
      # or for the user to be a Ruby coder.
      # - Sketchup#get_locale() returns a two-character code for most languages,
      # but four of them currently have five-character hyphenated codes.  The
      # longer codes are needed when there are multiple variants supported for
      # the same basic language.  For Chinese, this is still the case.  For
      # English and Portuguese there is only one variant currently supported by SU.
      # I don't know whether old versions of SU offered other variants or if the Trimble
      # developers anticipate possibly doing so in the future, so I stuck with
      # the long codes here rather than risk breaking on some untested combination
      # of locale and SU version.
      LANGUAGE_DECODE = {'en-US' => 'English',
                         'de'=>'Deutsch',
                         'es'=>'Español',
                         'fr'=>'Français',
                         'it'=>'Italiano',
                         'sv'=>'Svenska',
                         'pt-BR'=>'Português (Brasil)',
                         'ru'=>'Русский', # Russian
                         'ja'=>'日本語', # Japanese
                         'ko'=>'한국어', # Korean
                         'zh-CN'=>'中文 (简体)', # simplified Chinese
                         'zh-TW'=>'中文 (繁體)', # traditional Chinese
                         'nl'=>'Nederlands'
                        }  unless defined? LANGUAGE_DECODE
      # reverse lookup hash
      LANGUAGE_ENCODE = LANGUAGE_DECODE.invert unless defined? LANGUAGE_ENCODE
      def self.language=(language)
        stack=caller()
        tails = stack.map {|level| level.split("/")[-1]}
        tails = tails.join("\n***")
        idx = AVAILABLE_LANGUAGES.index(language)
        if idx.nil?
          # this shouldn't happen, but retained as a trap for coding errors
          UI.messagebox(language + DangLH[" not supported"])
          @language = "en-US"
        else
          @language = language
        end
        DangLH.parse_file(@language)
        # if we don't kill the WebDialogs, they retain the previous
        # contents!
        kill_wds()
      end
      def self.language
        @language
      end

      # diagnostic printout (not actively used by the code)
      def self.puts_params
        puts "Current User Parameters:"
        puts "arrow_style = #{@arrow_style} (#{@arrow_style.class})"
        puts "arrow_scale = #{@arrow_scale} (#{@arrow_scale.class})"
        puts "user_radius = #{user_radius} (#{@user_radius.class})"
        puts "arc_segments = #{@arc_segments} (#{@arc_segments.class})"
        puts "dim_line_scale = #{@dim_line_scale} (#{@dim_line_scale.class})"
        puts "text_scale = #{@text_scale} (#{@text_scale.class})"
        puts "user_text_height = #{@user_text_height} (#{@user_text_height.class})"
        puts "square90 = #{@square90} (#{@square90.class})"
        puts "language = ${self.language} (#{LANGUAGE_DECODE[self.language()]})"
        nil
      end

      # NOTE: ordering of these elements matters for consistency with the
      # WebDialog parameters.
      # return is an array of values
      def self.defaults
        # hard-wired constants as starting point
        defaults = [
          DEFAULT_ARROW_STYLE,
          DEFAULT_ARROW_SCALE,
          DEFAULT_RADIUS,
          DEFAULT_ARC_SEGMENTS,
          DEFAULT_LINE_SCALE,
          DEFAULT_TEXT_SCALE,
          DEFAULT_TEXT_HEIGHT,
          DEFAULT_SQUARE90,
          DEFAULT_LANGUAGE
        ]
        # this tries to read defaults saved as an encoded string
        # It requires the saved version to match the current version to avoid any
        # possibility of incompatible format, which can cause bugs due to mis-parsing values.
        defaults_version = Sketchup.read_default('JWMAngleDim', 'version')
        if (!defaults_version.nil?) && (defaults_version == SLB_JWMPlugins::AngularDim::EXTENSION_VERSION)
          saved_default_string = Sketchup.read_default('JWMAngleDim','defaults')
        else
          saved_default_string = nil
        end
        unless saved_default_string.nil?
          # we read saved defaults string ok - process it to array of values
          defaults = unescape_values(saved_default_string, true)
          # defaults[0] = arrow style stays as string
          defaults[1] = defaults[1].to_f #arrow scale
          defaults[2] = defaults[2].to_l #radius
          defaults[3] = defaults[3].to_i #arc segments
          defaults[4] = defaults[4].to_f #line scale
          defaults[5] = defaults[5].to_f #text scale
          defaults[6] = defaults[6].to_l #text height
          defaults[7] = defaults[7] #square90 checkbox
          # defaults[8] = language stays as string
          if defaults.length != 9
            # no previously saved language default.  This is only needed while
            # testing without updating EXTENSION_VERSION, since a variation in
            # version will ignore saved defaults and use the constants anyway.
            defaults[8] = DEFAULT_LANGUAGE
          end
        end
        defaults
      end

      def self.reset_defaults
        defs_array = self.defaults()
        self.arrow_style = defs_array[0]
        self.arrow_scale = defs_array[1]
        self.user_radius = defs_array[2]
        self.arc_segments = defs_array[3]
        self.dim_line_scale = defs_array[4]
        self.text_scale = defs_array[5]
        self.user_text_height = defs_array[6]
        self.square90 = defs_array[7]
        self.language = defs_array[8]
        nil
      end

      def self.reset_hard_defaults
        self.arrow_style = DEFAULT_ARROW_STYLE
        self.arrow_scale = DEFAULT_ARROW_SCALE
        self.user_radius = DEFAULT_USER_RADIUS
        self.arc_segments = DEFAULT_ARC_SEGMENTS
        self.dim_line_scale = DEFAULT_LINE_SCALE
        self.text_scale = DEFAULT_TEXT_SCALE
        self.user_text_height = DEFAULT_TEXT_HEIGHT
        self.square90 = DEFAULT_SQUARE90
        self.language = DEFAULT_LANGUAGE
      end

      # get to our saved values. This acts like an initializer for the class singleton variables
      self.reset_defaults()

      # so we can get at tool instance data when necessary...
      @active_tool = nil
      def self.active_tool=(tool)
        @active_tool = tool
      end
      def self.active_tool()
        @active_tool
      end

      # so Tool methods can get at the current settings dialog (which belongs to the class
      # not a Tool instance so that settings can be altered even when no AD Tool is active.
      # There is no corresponding setter because nothing outside this class should be able
      # to assign a dialog to @wd.
      def self.settings_dialog
        @wd
      end

      #---------------------------------
      # open the user input web dialog with current values displayed
      # The wdAngleDim_settings method expects lengths to use SU's
      # Length class to simplify formatting.
      # Note: language is handled by a separate inputbox via self.choose_language()
      def self.launch_input_dialog
        settings = [@arrow_style,
                    @arrow_scale,
                    @user_radius,
                    @arc_segments,
                    @dim_line_scale,
                    @text_scale,
                    @user_text_height,
                    @square90]
        self.wdAngleDim_settings(settings)
      end

      ###################################
      #let the user choose preferred translation language
      def self.choose_language
        # translate the available languages
        choices = []
        AVAILABLE_LANGUAGES.each {|code| choices << LANGUAGE_DECODE[code]}
        # Space padding on the Choose Language prompt to work around a silly behavior in UI.inputbox on Mac.
        # Without the padding it truncates at the first space in the prompt!
        # This may not be adequate in all languages!!
        choice = UI.inputbox([DangLH["Choose Language"]+"     "],
                             [LANGUAGE_DECODE[self.language()]],
                             [choices.join("|")],
                             DangLH["Choose a Language"])
        if choice
          # the returned value is from the list of choices, which are language long names
          # in native alphabet. Need to convert back to the short code
          code = LANGUAGE_ENCODE[choice[0]]
          if self.language != code
            # All dynamically managed text (dialogs, prompts, and status) will change language immediately,
            # but menu text and tooltips are frozen when SU creates the menu and
            # toolbar items.  They can only be changed by restarting SU.
            UI.messagebox(DangLH["You must restart SketchUp for the selected language to affect menu items and tooltips"])
          end
          self.language=code
          # save the new language as default for next session
          coded_defaults = self.escape_values(true, true)
          last_pipe_idx = coded_defaults.rindex('|')
          new_defaults = coded_defaults[0, last_pipe_idx + 1] << code
          Sketchup.write_default('JWMAngleDim', 'defaults', new_defaults)
          Sketchup.write_default('JWMAngleDim', 'version', EXTENSION_VERSION)
        end
      end

      ###################################
      # load the values from the inputs array into our attributes.
      # Elements of inputs array must be ordered correctly for
      # this to be correct (maybe a hash would have been smarter...).
      # This expects the caller to have preprocessed the values to put
      # them in a conventional form:
      #  - escape sequences for characters must be replaced by their original
      #    values.  e.g. &apos; -> ', &quot; -> "
      #  - strings representing floats must use period for the decimal separator.
      #    This is because Ruby's to_f does not know whether the user's locale
      #    uses commas or periods for the decimal.
      #  - strings representing lengths **must not** have commas changed to periods.
      #    This is because SketchUp's to_l **does** know the user's locale and
      #    expects strings to be formatted accordingly.
      def self.process_user_params(inputs)
        begin
          changed = false

          if inputs != false && !inputs.nil?
            if inputs[0] != self.arrow_style
              self.arrow_style = inputs[0]
              changed = true
            end
            if inputs[1].to_f != self.arrow_scale
              self.arrow_scale = inputs[1]
              changed = true
            end
            if inputs[2].to_l != self.user_radius
              self.user_radius = inputs[2]
              changed = true
            end
            if inputs[3].to_i != self.arc_segments
              self.arc_segments = inputs[3]
              changed = true
            end
            if inputs[4].to_f != self.dim_line_scale
              self.dim_line_scale = inputs[4]
              changed = true
            end
            if inputs[5].to_f != self.text_scale
              self.text_scale = inputs[5]
              changed = true
            end
            if inputs[6].to_l != self.user_text_height
              self.user_text_height = inputs[6]
              changed = true
            end
            if inputs[7] != self.square90
              self.square90 = inputs[7]
              changed = true
            end
          end
        rescue Exception => e
          UI.messagebox("Bad input value: #{e.message}")
          # all we can do is tell the user and then wait again for callback from javascript
        end

        # When possible, tell the Tool to redraw the last AD
        tool = active_tool()
        if tool.nil?
          # we get here if the Tool is not active when the user modifies a setting.
          # The class will retain the new settings, but there is no Tool that can
          # possibly redraw anything so don't try. Trap nil here so we don't
          # get exceptions below.
        elsif changed
          if tool.drawn?
            # undo the previous operation before redrawing
            # this avoids buildup of obsolete operations on the
            # undo stack
            Sketchup.undo if tool.state==0
            tool.draw_angle_dim
          end
          # synchronize with the tool's displayed status
          tool.show_status
        end # tool.nil?
      end # process_user_params

      # test for whether user's locale uses comma as the decimal point
      def self.lengths_use_comma?
        begin
          # this will raise an Exception when "," is the decimal, not "."
          "0.0".to_l
          # no Exception, so "." is legal
          use_comma = false
        rescue
          # when we get the Exception, they are not using "." so must be ","
          use_comma = true
        end
      end

    end # DrawAngleDimTool
  end # AngularDim
end # SLB_JWMPlugins
