# Custom substitute for SketchUp's LanguageHandler to allow changing the translation
# language under control of our extension.
#
# Copyright 2017 SLBaumgartner
#
# File parsing code copied directly from SketchUp's langhandler.rb
# Copyright 2013, Trimble Navigation Limited
#
# This uses the same .strings file format as the original LanguageHandler.



module SLB_JWMPlugins
  module AngularDim
    class StringTranslator
      def initialize(language_code = "en-US")
        parse_file(language_code)
        @language_code = language_code.dup
      end

      def language_code()
        @language_code.dup
      end

      def [] (key)
        unless @lookup.nil?
          # we have a lookup hash, try the key
          trans = @lookup[key]
          if trans.nil?
            # key not found in the hash, return the key unchanged
            AD_Logger.log_message("StringTranslator for #{@language_code} has no match for:\n\t\"#{key}\"")
            trans = key
          end
        else
          # no lookup hash, return the key unchanged
          AD_Logger.log_message("StringTranslator has no lookup hash for #{language_code}") unless language_code == "en-US"
          trans = key
        end
        trans.dup
      end

      def parse_file(language_code)
        filepath = File.join(SLB_JWMPlugins::AngularDim::RESPATH, language_code, "draw_angle_dim.strings")
        @lookup = {} # discard previous hash, start out empty
        # parsing logic lifted directly from LanguageHandler.rb so the file format can be the same
        begin
          File.open(filepath, "r") do |file|
            entry_string = ''
            in_comment_block = false
            file.each_line { |line|
              # Ignore simple comment lines - BIG assumption that the whole line
              # is a comment.
              if !line.include?('//')
                # Also ignore comment blocks.
                if line.include?('/*')
                  in_comment_block = true
                end
                if in_comment_block
                  if line.include?('*/')
                    in_comment_block = false
                  end
                else
                  entry_string += line
                end
              end

              if entry_string.include?(';')
                # Parse the string into key and value parts.
                pattern = /\s*"(.+)"="(.+)"\s*;\s*/
                result = pattern.match(entry_string)
                if result && result.size == 3
                  key = result[1]
                  value = result[2]
                  @lookup[key] = value
                end
                entry_string = ''
              end
            } # each line
          end # File.open do  block
          # if we got here, the parsing succeeded, so remember the language
          @language_code = language_code
        rescue Errno::ENOENT
          unless language_code == "en-US"
            UI.messagebox("StringTranslator:\ntranslation file not found for language code #{language_code}\nusing English")
            AD_Logger("******translation file not found for #{language_code}")
            @language_code = "en-US"
          end
          @lookup = nil # file wasn't found (doesn't matter for en-US default because keys are also the translations)
        rescue Exception => e
          # some other error while processing the file contents
          UI.messagebox("StringTranslator error while parsing file:\n#{e.message}")
          AD_Logger("******StringTranslator error while parsing file:\n#{e.message}")
          # TBD: should we flush everything or keep what we've got so far?
        end
        nil
      end # parse_file
    end
  end
end
