# wdAngleDim_settings.rb.
# Code to set up a WebDialog input form and send/receive values to/from it
# MIT licence
# Steve Baumgartner, John McClenahan

# Version 6.1 2017-02-23

# This code started from Martin Rinehart's example WebDialog code
#   but has been VERY much extended by SLB

require 'sketchup.rb'


module SLB_JWMPlugins
  module AngularDim

    class DrawAngleDimTool

      ###############################
      # Create Help WebDialog
      def self.wdAngleDim_help
        if @wdhelp.nil?
        # Parameters are:
        #    dialogue title, scrollable?, prefs keyname, width, height, left, top, resizeable?
          @wdhelp = UI::WebDialog.new(
            "Angular Dimension Help", true, "WebDialog_help_v5", 500, 900, 650, 150, true)
          # Set path and filename of WebDialog page to open
          pathname = File.join(SLB_JWMPlugins::AngularDim::RESPATH, self.language(), 'wdAngleDim_help.html')
          @wdhelp.set_file( pathname  )

          @wdhelp.add_action_callback("close") do |js_wd, message|
            @wdhelp.close
          end

          @wdhelp.add_action_callback("fix_decimals") do |js_wd, message|
            if self.lengths_use_comma?
              js_wd.execute_script('lengths_use_commas();')
            end
            js_wd.execute_script('preparePage();')
          end
        end
        if (RUBY_PLATFORM.downcase =~ /mswin|mingw/) != nil ### PC
          mac=false
        else #MAC
          mac=true
        end
        if mac
          @wdhelp.show_modal()
        else
          @wdhelp.show()
        end # end if
      end

      ###########################
      # Create WebDialog and put values into it
      def self.wdAngleDim_settings(sendValues)
        # @wd retains the settings WebDialog once created
        if @wd.nil?
          # starting afresh, create a new WebDialog
          # Parameters are:
          #   dialogue title, scrollable?, prefs keyname, width, height, left, top, resizeable?
          @wd=UI::WebDialog.new(
            "Angular Dimension settings", true, "WebDialog_v5", 500, 580, 150, 150, true )

          # This callback function is triggered when user clicks WebDialog OK button
          # hits return in a field, or changes the arrow style selector.
          # JS sends the current values in the message.
          @wd.add_action_callback("get_inputs") do |js_wd, message|
            result = self.unescape_values(message)
            conv = result.join("\n")
            # last element tells us whether to close the WebDialog
            # not actually one of the settings parameters as such
            flag = result.pop

            process_user_params(result)

            case flag # flag value returned at end of text string from HTML page
            when 'OK'
              # user pressed ok, close the dialog
              js_wd.close()
            # release this reference so it doesn't persist
            when 'defaults'
              js_wd.execute_script('save_defaults_2();')
            when 'click'
              # Need a round-trip back to js to give it properly formatted
              # versions of the values we just received.
              pass_values_to_javascript(js_wd)
            end
          end

          # This callback function responds to JS request to send current values from Ruby code
          # and enter them into the WebDialog input boxes
          # Replace the currentValues string with your own values from your Ruby plugin
          # Make sure to encode foot and inch marks (single and double quotes) with
          #  HTML equivalents before sending them
          @wd.add_action_callback("set_values") do |js_wd, message|
            pass_values_to_javascript(js_wd)
            if self.lengths_use_comma?
              js_wd.execute_script('lengths_use_comma();')
              js_wd.execute_script('commaFormatPrompts();')
            end
          end

          # This callback function reads stored defaults from SketchUp and makes them active in the
          # Tool.
          @wd.add_action_callback('load_defaults') do |js_wd, message|
            # this makes the defaults active in the Tool Class's attributes
            process_user_params(self.defaults)
            # and this synchronizes them with javascript
            pass_values_to_javascript(js_wd)
          end

          # This callback function saves the current parameter values as defaults in SketchUp
          # The are saved as a single '|' joined String of attribute value Strings.
          # Floats always use '.' as decimal because they are Ruby Floats.
          # Lengths will use ',' as decimal if the user's locale does.
          @wd.add_action_callback('save_defaults') do |js_wd, message|
            Sketchup.write_default('JWMAngleDim', 'defaults', self.escape_values(false, false))
            Sketchup.write_default('JWMAngleDim', 'version', EXTENSION_VERSION)
          end

          @wd.add_action_callback('show_help') do |js_wd, message|
            wdAngleDim_help()
          end

          # take the help dialog down along with this one.
          @wd.set_on_close do
            @wdhelp.close unless @wdhelp.nil?
          end

          # Set path and filename of WebDialog page to open
          pathname = File.join(SLB_JWMPlugins::AngularDim::RESPATH, self.language(), 'wdAngleDim_settings.html')
          @wd.set_file( pathname  )
        end # if @wd.nil?

        if (RUBY_PLATFORM.downcase =~ /mswin|mingw/) != nil ### PC
          mac=false
        else #MAC
          mac=true
        end
        if mac
          @wd.show_modal()
        else
          @wd.show()
        end # end if
      end    # end of wdAngleDim_settings()

      # kill of the WebDialogs so that they don't retain stale values
      # Needed, for example, if we change language
      def self.kill_wds
        if ! @wd.nil?
          @wd.close
        end
        @wd = nil
        if ! @wdhelp.nil?
          @wdhelp.close
        end
        @wdhelp = nil
      end

      def self.pass_values_to_javascript(wd)
        sendValues = escape_values(false)
        set_result = 'set_inputs("'  + sendValues + '");'
        wd.execute_script( set_result );
      end

      # test for whether user's locale uses comma as the decimal point
      def self.lengths_use_comma?
        begin
          # this will raise an Exception when "," is the decimal, not "."
          "0.0".to_l
          # no Exception, so "." is legal
          use_comma = false
        rescue
          # when we get the Exception, they are not using "." so must be ","
          use_comma = true
        end
      end

      # Escape foot (') and inch (") characters found in Imperial length strings before passing to JS.
      # Otherwise they cause issues with parsing of strings in JS
      # Also handle conversion to comma decimal points if used in this locale.
      # convert_commas = false turns off this formatting so that we can force
      # values to always use '.'
      def self.escape_values(use_class_defaults = false, convert_commas = true)
        if use_class_defaults
          values = self.defaults().map{|elt| elt.to_s}.join('|')
        else
          values = "#{self.arrow_style}|#{self.arrow_scale}|#{self.user_radius}|#{self.arc_segments}|#{self.dim_line_scale}|#{self.text_scale}|#{self.user_text_height}|#{self.square90}|#{self.language}"
        end
        values = values.gsub("'", '&apos;')
        values = values.gsub('"', '&quot;')
        values = values.gsub("  ", " ")
        if self.lengths_use_comma?
          if convert_commas
            # format for locale decimal convention
            # Float.to_s doesn't pay attention to locale, it always formats using '.', so need
            # to convert.  Lengths will already be formatted for locale, so they shouldn't contain
            # '.'.  Other fields also should not contain '.' in any case.  So the gsub should only
            # affect the fields for Floats.
            values = values.gsub(".", ",")
          else
            # force to '.' decimal convention
            # Floats will already use '.'.  Lengths will use ',' because they are locale sensitive.
            # Other fields shouldn't contain either '.' or ','.  So translation of ',' to '.'
            # should serve only to affect decimal valued Lengths
            values = values.gsub(",", ".")
          end
        end
        values
      end

      ##################################
      # encoded_values is a '|' joined String of values that have single and double
      # quote characters escaped and that has decimal place represented according to
      # user's locale setting.  This method undoes the escape sequences, splits the
      # String into an Array, and then replaces commas with periods in Float-targeted
      # elements but not in Length-targeted ones when the locale uses comma for decimals.
      # When from_defaults is true, we also convert Length-trageting fields because
      # we always store defaults using '.' decimals to avoid issues if the user changes
      # locale.
      # It returns the Array of decoded Strings, deferring conversion to numbers to
      # the caller's code.
      def self.unescape_values(encoded_values, from_defaults = false)
        decoded = encoded_values.gsub('&apos;', "'")
        decoded = decoded.gsub('&quot;', '"')
        decoded = decoded.gsub(" ", "  ")
        value_array = decoded.split("|")
        if self.lengths_use_comma?
          # convert elements that target Floats
          value_array[1] = value_array[1].gsub(",", ".")
          value_array[4] = value_array[4].gsub(",", ".")
          value_array[5] = value_array[5].gsub(",", ".")
          # if from defaults, Lengths will need conversion *to* comma decimals
          # else to_l will fail!
          if from_defaults
            value_array[2] = value_array[2].gsub(".", ",")
            value_array[6] = value_array[6].gsub(".", ",")
          end
        end
        joined=value_array.join("\n")
        value_array
      end
    end # DrawAngleDimTool
  end # AngularDim
end # SLB_JWMPlugins

